import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:wyatt_notification_bloc/src/data/models/remote_notification_wrapper_model.dart';
import 'package:wyatt_notification_bloc/src/domain/data_sources/remote/notification_remote_data_source.dart';
import 'package:wyatt_notification_bloc/src/domain/entities/remote_notifications.dart';

class FirebaseMessagingDataSourceImpl extends NotificationRemoteDataSource {
  final _instance = FirebaseMessaging.instance;

  @override
  Future<String> getToken() async {
    final token = await _instance.getToken();
    if (token != null && token.isNotEmpty) {
      return token;
    }
    throw Exception('error on token');
  }

  @override
  Stream<RemoteNotificationWrapper> onNotification() =>
      FirebaseMessaging.onMessage
          .map(RemoteNotificationWrapperModel.fromFirebaseModel);

  @override
  Stream<RemoteNotificationWrapper> onNotificationOpened() =>
      FirebaseMessaging.onMessageOpenedApp
          .map(RemoteNotificationWrapperModel.fromFirebaseModel);

  @override
  Future<RemoteNotificationWrapper?> getInitialMessage() =>
      _instance.getInitialMessage().then<RemoteNotificationWrapper?>(
            RemoteNotificationWrapperModel.fromFirebaseModel,
          );

  @override
  Future<void> register() async {
    await _instance.requestPermission();
    await FirebaseMessaging.instance
        .setForegroundNotificationPresentationOptions(
      alert: true, // Required to display a heads up notification
      badge: true,
      sound: true,
    );
  }

  @override
  Future<void> subscribeToTopic(String topic) =>
      _instance.subscribeToTopic(topic);

  @override
  Future<void> unsubscribeFromTopic(String topic) =>
      _instance.unsubscribeFromTopic(topic);
}
